# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""correcting cv carta env variables

Revision ID: b48b587be56b
Revises: bdd8bc5e2960
Create Date: 2021-12-16 15:10:58.354422

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "b48b587be56b"
down_revision = "bdd8bc5e2960"
branch_labels = None
depends_on = None

script = b"""#!/bin/sh
set -o errexit

export HOME=$_CONDOR_SCRATCH_DIR
{{#remote-cv}}
export CAPO_PATH=$(readlink -f $_CONDOR_SCRATCH_DIR)
export CAPO_PROFILE={{profile}}
{{/remote-cv}}
fetch_foundation(){
  ../productfetcher{{product_locator}}

  echo "Moving images out of subdirectories...."
  for dir in *.fits; do mv $dir $dir.1; done;

  find . -mindepth 1 -type f -print -exec mv {} . \\;
  find . -mindepth 1 -type d -empty -delete
  echo "Done. Ready for Carta!"
}


{{#need_data}}
./framework.sh -r .
chmod 770 .
cd rawdata/

fetch_foundation {{^single_image}}&{{/single_image}}

.{{/need_data}}./carta_envoy {{#need_data}}--parallel{{/need_data}} {{#single_image}}--single -i {{imageName}}{{/single_image}} -d $1 -n $2 -wf $3 -r $4
{{#need_data}}wait{{/need_data}}

"""

old_script = b"""#!/bin/sh
set -o errexit

export HOME=$TMPDIR
{{#remote-cv}}
export CAPO_PATH=$(readlink -f $TMPDIR)
export CAPO_PROFILE={{profile}}
{{/remote-cv}}
fetch_foundation(){
  ../productfetcher{{product_locator}}

  echo "Moving images out of subdirectories...."
  for dir in *.fits; do mv $dir $dir.1; done;

  find . -mindepth 1 -type f -print -exec mv {} . \\;
  find . -mindepth 1 -type d -empty -delete
  echo "Done. Ready for Carta!"
}


{{#need_data}}
./framework.sh -r .
chmod 770 .
cd rawdata/

fetch_foundation {{^single_image}}&{{/single_image}}

.{{/need_data}}./carta_envoy {{#need_data}}--parallel{{/need_data}} {{#single_image}}--single -i {{imageName}}{{/single_image}} -d $1 -n $2 -wf $3 -r $4
{{#need_data}}wait{{/need_data}}

"""


def upgrade():
    conn = op.get_bind()
    conn.execute(
        f"""
        UPDATE workflow_templates
        SET content=%s WHERE filename='carta.sh'
        """,
        script,
    )


def downgrade():
    conn = op.get_bind()
    conn.execute(
        f"""
        UPDATE workflow_templates
        SET content=%s WHERE filename='carta.sh'
        """,
        old_script,
    )
