# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""add errexit to std cms imaging

Revision ID: ecb4acbecbce
Revises: 2ac701610fc7
Create Date: 2021-07-02 15:04:25.760177

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "ecb4acbecbce"
down_revision = "2ac701610fc7"
branch_labels = None
depends_on = None


script_content = """#!/bin/sh

export HOME=$TMPDIR

set -o errexit
./framework.sh -d .
chmod 770 .
mv {{sdmId}}.ms ./rawdata
./casa_envoy --standard-img $1 $2
./conveyor --deliver-img $1

"""


def upgrade():
    op.execute(
        f"""
    UPDATE workflow_templates
    SET content=E'{script_content}' WHERE filename='std_cms_imaging.sh'
    """
    )


def downgrade():
    op.execute(
        f"""
    UPDATE workflow_templates
    SET content=E'{old_script_content}' WHERE filename='std_cms_imaging.sh'
    """
    )


old_script_content = """#!/bin/sh

export HOME=$TMPDIR

./framework.sh -d .
chmod 770 .
mv {{sdmId}}.ms ./rawdata
./casa_envoy --standard-cmsimage $1 $2
./conveyor --deliver-image $1

"""
